/**
 * FlAIR - Flickr Application for Adobe AIR
 * IMAGE.JS - FlickrImage Class
 * instantiates new images, adding them to the thumbnail view, and handling image events
 *  
 *   http://farm{farm-id}.static.flickr.com/{server-id}/{id}_{secret}.jpg
 *   http://farm{farm-id}.static.flickr.com/{server-id}/{id}_{secret}_[mstb].jpg
 *   http://farm{farm-id}.static.flickr.com/{server-id}/{id}_{o-secret}_o.(jpg|gif|png)
 */

var FlickrImage = new Class({
	initialize:function(xml){
		this._xml = xml;

		var a = new Element('a',{
			'href': this.buildURL(FlickrImageSize.ORIGINAL),
			'rel':'thumbs'
		});
		var img = new Element('img',{
			'height':75,
			'width':75,
			'src': this.buildURL(FlickrImageSize.SQUARE)
		});
		img.onclick = this.viewLarge.bind(this);
		img.onload = function()
		{
			img.effect('opacity',{duration:300}).start(0,.3);
		}
		img.onmouseover = function(){
			img.effect('opacity',{duration:300}).start(.3,1);
		}
		img.onmouseout = function(){
			img.effect('opacity',{duration:300}).start(1,.3);
		}
		
		var dragImage =new Image();
        dragImage.src = "img/go.png";
				
		img.ondragstart=function(event){

			event.dataTransfer.setData('text/plain', 'testing');
			// event.dataTransfer.allowedActions = "copy";
			event.dataTransfer.setDragImage(dragImage, 24, 24);
			event.stopPropagation();
		}
		var div = new Element('div');
		img.injectInside(a);
		a.injectInside(div);
		img.setStyle('opacity', 0);

		div.injectInside('thumbnails');
	},
	buildURL:function(size)
	{
		var xml 	= this._xml;
		var farmid 	= xml.getAttribute('farm');
		var serverid = xml.getAttribute('server');
		var id 		= xml.getAttribute('id');

		if(!xml.getAttribute('originalsecret') && size == FlickrImageSize.ORIGINAL)	{
			size = FlickrImageSize.MEDIUM;
		}
			
		var secret 	= size == FlickrImageSize.ORIGINAL ? xml.getAttribute('originalsecret') : xml.getAttribute('secret');
		var ext 	= size == FlickrImageSize.ORIGINAL ? xml.getAttribute('originalformat') : 'jpg';
		if(size) size = "_" + size;

		return "http://farm" + farmid + ".static.flickr.com/" + serverid + "/" + id + "_" + secret + size +  "." + ext;
	},
	viewLarge:function(evt)
	{
		evt.stopPropagation();
		evt.preventDefault();
		var xml = this._xml;
		// take the available width and minus 200 for metadata
		var val = Math.min(nativeWindow.height, nativeWindow.width) - 240; 
		
		
		// try and determine size
		var size = val > 1024 ? FlickrImageSize.ORIGINAL : (val > 240 ? FlickrImageSize.MEDIUM : FlickrImageSize.THUMB);
		var dialog = new Dialog('ui-photo',{screenOpacity:0.6});
		
		// clear out existing content
		$E('.img', dialog.element).innerHTML ='';
		$E('h2', dialog.element).innerHTML = '';
		$E('.description', dialog.element).innerHTML = '';
				
		if(this._fullXML)
		{
			var func = injectContent.bind(this);
			func(this._fullXML);

		}else{
			app.user.flickr.api('flickr.photos.getInfo',{params:{photo_id:xml.getAttribute('id')}, success:injectContent.bind(this)});
		}

		function injectContent(xml)
		{
		    this._fullXML = xml.getElementsByTagName('photo')[0];
	
			var shell = $E('.img', dialog.element);
			var img = new Element('img',
			{
				'src': this.buildURL(size)
			});
			
			img.setStyle('opacity','0');
			img.onload = function(){ 
				dialog.show();
				dialog.resize({width: img.width + 240, height: img.height + 20});
				img.effect('opacity',{duration:500}).start(0,1);
			};
			img.oncontextmenu = function(evt)
			{
				var menu = new air.NativeMenu();
				evt.preventDefault();
				var command = menu.addItem(new air.NativeMenuItem('Set as Background'));
				command.addEventListener(air.Event.SELECT, function(){  
					app.setEncrypted('background' + app.user.name, img.src);
					document.body.style.background = '#0D0D0D url(' + img.src + ') no-repeat 50% 50%';
		
					});
				menu.display(nativeWindow.stage, evt.clientX, evt.clientY);
				return false;
			}
			img.injectInside(shell);
			
			$E('.phototitle', dialog.element).innerHTML = xml.getElementsByTagName('title')[0].textContent;			
			$E('.description', dialog.element).innerHTML = xml.getElementsByTagName('description')[0].textContent;
			
			function newItem(label, callback)
			{
				
			}
		}

	}
});

var FlickrImageSize = {
	getSize:function(size)
	{
		if(FlickrImageSize.SQUARE == size) return 75;
		if(FlickrImageSize.THUMB == size) return 100;
		if(FlickrImageSize.SMALL == size) return 240;
		if(FlickrImageSize.MEDIUM == size) return 500;
		if(FlickrImageSize.BIG == size) return 1024;
		if(FlickrImageSize.ORIGINAL == size) return -1;
	}
};
FlickrImageSize.SQUARE = 's'; // square, 75x75
FlickrImageSize.THUMB = 't'; // thumbnail, 100 on longest side 
FlickrImageSize.SMALL = 'm'; // small, 240 on longest side
FlickrImageSize.MEDIUM = ''; // medium, 500 on longest side
FlickrImageSize.BIG = 'b'; // big, 1024 on longest side
FlickrImageSize.ORIGINAL = 'o'; // original

